@@ -62,7 +62,9 @@ typedef struct Syntax_Table_Type
    struct Syntax_Table_Type *next;     /* pointer to next table */
 #if JED_HAS_DFA_SYNTAX
    struct Highlight *hilite;
+   SLang_Name_Type *init_dfa_callback;
 #endif
+   int use_dfa_syntax;
    unsigned char fortran_comment_chars[256];
 }
 Syntax_Table_Type;
@@ -84,18 +86,11 @@ Syntax_Table_Type;
 
 extern Syntax_Table_Type *Default_Syntax_Table;
 
-extern int find_matching_delimiter (int *);
-extern int parse_to_point (void);
-extern void use_syntax_table (char *);
-extern void create_syntax_table (char *);
-extern void define_syntax (int *, char *);
 extern void init_syntax_tables (void);
 extern void blink_match(void);
 extern int goto_match(void);
-extern void set_syntax_flags (char *, int *);
-extern void define_keywords (char *, char *, int *, int *);
-extern Syntax_Table_Type *find_syntax_table (char *, int);
-extern void jed_set_fortran_comment_style (char *, char *);
+extern Syntax_Table_Type *jed_find_syntax_table (char *, int);
+extern int jed_init_syntax (void);
 
 #if JED_HAS_LINE_ATTRIBUTES
 extern void jed_syntax_parse_buffer (int);
