@@ -238,7 +238,8 @@ int jed_prepare_for_modification (void)
    if (0 == CBuf->file[0])
      return 0;			       /* not attached to a file */
    
-   if (SLang_Error == 0)
+   if ((SLang_Error == 0)
+       && (0 == (CBuf->flags & BUFFER_NON_LOCKING)))
      {
 	check_buffer (CBuf);
 	if (CBuf->flags & FILE_MODIFIED)
@@ -246,6 +247,15 @@ int jed_prepare_for_modification (void)
 	     if (1 != jed_get_y_n ("File changed on disk.  Do you really want to edit this buffer"))
 	       return -1;
 	  }
+	if (1 == jed_buffer_file_is_readonly (CBuf))
+	  {
+	     if (1 != jed_get_y_n ("Disk file is read-only.  Do you really want to edit this buffer"))
+	       {
+		  CBuf->flags |= READ_ONLY;
+		  return -1;
+	       }
+	  }
+
 	(void) jed_lock_buffer_file (CBuf);
      }
 
@@ -326,9 +336,6 @@ int jed_deln (int n) /*{{{*/
 
    if ((n == 0) || !CLine->len) return 0;
 
-   if (-1 == jed_prepare_for_modification ())
-     return -1;
-
    nn = CLine->len - 1;
    p = CLine->data + nn;
    if ((*p == '\n') && (CBuf != MiniBuffer)) nn = nn - Point; else nn = nn - Point + 1;
@@ -337,6 +344,10 @@ int jed_deln (int n) /*{{{*/
 
    nn = nn > n ? n : nn;
    if (!nn) return (0);
+
+   if (-1 == jed_prepare_for_modification ())
+     return -1;
+
    jed_update_marks(CDELETE, nn);
    record_deletion(p, nn);
    CLine->len -= nn;
