@@ -593,13 +593,14 @@ static void insert_string (void)
    char *s;
    SLang_BString_Type *bs;
    unsigned int len;
+   int ret = 0;
 
    switch (SLang_peek_at_stack ())
      {
       case SLANG_STRING_TYPE:
 	if (-1 == SLang_pop_slstring (&s))
 	  return;
-	(void) jed_insert_string (s);
+	ret = jed_insert_string (s);
 	SLang_free_slstring (s);
 	break;
 	
@@ -607,11 +608,15 @@ static void insert_string (void)
 	if (-1 == SLang_pop_bstring (&bs))
 	  return;
 	if (NULL != (s = (char *)SLbstring_get_pointer (bs, &len)))
-	  jed_ins_chars ((unsigned char *)s, len);
+	  ret = jed_ins_chars ((unsigned char *)s, len);
 	SLbstring_free (bs);
      }
+   
+   if (ret == -1)
+     jed_verror ("insert: insertion failed");
 }
 
+
 static void get_key_binding (void) 
 {
    unsigned char *s = NULL;
@@ -691,7 +696,7 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_I("set_mode", intrin_set_mode, VOID_TYPE),
    MAKE_INTRINSIC_S("buffer_visible", jed_buffer_visible, INT_TYPE),
    MAKE_INTRINSIC("exit_jed", exit_jed, VOID_TYPE, 0),
-   MAKE_INTRINSIC_SSS("set_color", set_term_colors, VOID_TYPE),
+   MAKE_INTRINSIC_SSS("_set_color", set_term_colors, VOID_TYPE),
    MAKE_INTRINSIC_S("color_number", map_color_object_to_number, INT_TYPE),
 
 #ifndef IBMPC_SYSTEM
@@ -797,7 +802,7 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC("quit_jed", quit_jed, VOID_TYPE, 0),
    MAKE_INTRINSIC_S("read_file", find_file_cmd, INT_TYPE),
    MAKE_INTRINSIC_4("read_with_completion", read_object_with_completion, VOID_TYPE, STRING_TYPE, STRING_TYPE, STRING_TYPE, INT_TYPE),
-   MAKE_INTRINSIC_I("set_abort_char", set_abort_char, VOID_TYPE),
+   MAKE_INTRINSIC_I("set_abort_char", jed_set_abort_char, VOID_TYPE),
    MAKE_INTRINSIC("suspend", sys_spawn_cmd, VOID_TYPE, 0),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, VOID_TYPE),
    MAKE_INTRINSIC_S("buffer_keystring", do_buffer_keystring, VOID_TYPE),
@@ -827,18 +832,10 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_II("random", make_random_number, INT_TYPE),
    MAKE_INTRINSIC("translate_region", translate_region, VOID_TYPE, 0),
    MAKE_INTRINSIC_S("set_current_kbd_command", set_current_kbd_command, VOID_TYPE),
-   MAKE_INTRINSIC_I("find_matching_delimiter", find_matching_delimiter, INT_TYPE),
    MAKE_INTRINSIC("blink_match", blink_match, VOID_TYPE, 0),
-   MAKE_INTRINSIC("parse_to_point", parse_to_point, INT_TYPE, 0),
-   MAKE_INTRINSIC_SI("set_syntax_flags", set_syntax_flags, VOID_TYPE),
-   MAKE_INTRINSIC_IS("define_syntax", define_syntax, VOID_TYPE),
-   MAKE_INTRINSIC_S("use_syntax_table", use_syntax_table, VOID_TYPE),
-   MAKE_INTRINSIC_S("create_syntax_table", create_syntax_table, VOID_TYPE),
-   MAKE_INTRINSIC_4("define_keywords_n", define_keywords, VOID_TYPE, STRING_TYPE, STRING_TYPE, INT_TYPE, INT_TYPE),
    MAKE_INTRINSIC_S("set_expansion_hook", set_expansion_hook, VOID_TYPE),
    MAKE_INTRINSIC("clear_message", clear_message, VOID_TYPE, 0),
    MAKE_INTRINSIC("flush_input", flush_input, VOID_TYPE, 0),
-   MAKE_INTRINSIC_SS("set_fortran_comment_chars", jed_set_fortran_comment_style, SLANG_VOID_TYPE),   
    MAKE_INTRINSIC_0("get_word_chars", jed_get_word_chars, STRING_TYPE),
    MAKE_INTRINSIC_SI("core_dump", exit_error_cmd, VOID_TYPE),
    MAKE_INTRINSIC("get_last_macro", get_last_macro, VOID_TYPE, 0),
@@ -881,6 +878,7 @@ static SLang_Intrin_Fun_Type Jed_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_I("send_process_eof", jed_send_process_eof, VOID_TYPE),
    MAKE_INTRINSIC_I("get_process_input", get_process_input, VOID_TYPE),
    MAKE_INTRINSIC_II("signal_process", jed_signal_process, VOID_TYPE),
+   MAKE_INTRINSIC_II("signal_fg_process", jed_signal_fg_process, VOID_TYPE),
    MAKE_INTRINSIC_II("process_query_at_exit", jed_query_process_at_exit, VOID_TYPE),
 #endif
 
@@ -980,10 +978,10 @@ static SLang_Intrin_Var_Type Jed_Variables [] =
    MAKE_VARIABLE("_jed_version_string", &Jed_Version_String, STRING_TYPE, 1),
    MAKE_VARIABLE("_jed_secure_mode", &Jed_Secure_Mode, INT_TYPE, 1),
 
+   MAKE_VARIABLE("Simulate_Graphic_Chars", &Jed_Simulate_Graphic_Chars, INT_TYPE, 0),
 #ifndef IBMPC_SYSTEM
    MAKE_VARIABLE("OUTPUT_RATE", &Obsolete_Int_Variable, INT_TYPE, 0),
    MAKE_VARIABLE("USE_ANSI_COLORS", &tt_Use_Ansi_Colors, INTP_TYPE, 0),
-   MAKE_VARIABLE("Simulate_Graphic_Chars", &Jed_Simulate_Graphic_Chars, INT_TYPE, 0),
    MAKE_VARIABLE("TERM_CANNOT_INSERT", &tt_Term_Cannot_Insert, INTP_TYPE, 0),
    MAKE_VARIABLE("TERM_CANNOT_SCROLL", &tt_Term_Cannot_Scroll, INTP_TYPE, 0),
    MAKE_VARIABLE("TERM_BLINK_MODE", &tt_Blink_Mode, INTP_TYPE, 0),
@@ -991,7 +989,7 @@ static SLang_Intrin_Var_Type Jed_Variables [] =
 
 #ifdef IBMPC_SYSTEM
    MAKE_VARIABLE("Case_Sensitive_Filenames", &Jed_Filename_Case_Sensitive, INT_TYPE, 0),
-# if defined(__MSDOS_16BIT__)
+# if defined(__MSDOS_16BIT__) || defined(__WIN32__)
    MAKE_VARIABLE("NUMLOCK_IS_GOLD", &NumLock_Is_Gold, INT_TYPE, 0),
 # endif
 # ifndef MSWINDOWS
@@ -1011,7 +1009,9 @@ static SLang_Intrin_Var_Type Jed_Variables [] =
    MAKE_VARIABLE("KILL_ARRAY_SIZE", &Kill_Array_Size, INT_TYPE, 1),
 #endif
    MAKE_VARIABLE("_Backspace_Key", &_Jed_Backspace_Key, STRING_TYPE, 1),
-
+#if JED_HAS_TTY_MENUS
+   MAKE_VARIABLE("Menus_Active", &Jed_Menus_Active, INT_TYPE, 1),
+#endif
    MAKE_VARIABLE(NULL, NULL, 0, 0)
 };
 
