@@ -10,6 +10,10 @@
 
 #include <stdio.h>
 
+#if defined(__QNX__) && defined(__WATCOMC__)
+# include <env.h>
+#endif
+
 #if JED_HAS_SUBPROCESSES
 /* Everything else here is in this '#if' */
 
@@ -69,6 +73,8 @@
 
 /*}}}*/
 
+#define MAX_TTY_SLAVE_NAME	256
+
 int Num_Subprocesses;
 int Max_Subprocess_FD;
 
@@ -235,6 +241,23 @@ int jed_signal_process (int *fd, int *sig) /*{{{*/
    return 0;
 }
 
+int jed_signal_fg_process (int *fd, int *sig) /*{{{*/
+{
+   int pid;
+
+   Process_Type *p;
+   if (NULL == (p = get_process (*fd))) return -1;
+
+#ifdef HAVE_TCGETPGRP
+   pid = tcgetpgrp (p->rd);
+   if (pid == -1)
+#endif
+     pid = p->pid;
+
+   (void) kill (-pid, *sig);
+   return 0;
+}
+
 /*}}}*/
 #endif
 
@@ -415,11 +438,13 @@ static void child_signal_handler (int sig) /*{{{*/
 	     return_status = WTERMSIG (status);
 	     status = PROCESS_SIGNALLED;
 	  }
+#ifdef WIFSTOPPED
 	else if (WIFSTOPPED (status))
 	  {
 	     return_status = WSTOPSIG(status);
 	     status = PROCESS_STOPPED;
 	  }
+#endif
 	/* What else?? */
 	
 	p = Processes;
@@ -516,7 +541,7 @@ static int open_process (char *pgm, char **argv) /*{{{*/
    int pid, i;
    Process_Type *p;
    SLang_MMT_Type *mmt;
-   char slave_tty_name [80];
+   char slave_tty_name [MAX_TTY_SLAVE_NAME];
    int max_write_tries;
    SLSig_Fun_Type *sig_orig[NSIG];
 
@@ -715,7 +740,7 @@ static int open_process (char *pgm, char **argv) /*{{{*/
    int pid, i;
    Process_Type *p;
    SLang_MMT_Type *mmt;
-   char slave_tty_name [80];
+   char slave_tty_name [MAX_TTY_SLAVE_NAME];
 
    if (Jed_Secure_Mode)
      {
