@@ -97,6 +97,8 @@ extern unsigned _stklen = 10000U;
 /*}}}*/
 
 int Jed_Secure_Mode;
+int Jed_Load_Quietly = 0;
+
 char Jed_Root_Dir [JED_MAX_PATH_LEN];
 
 typedef struct /*{{{*/
@@ -130,7 +132,6 @@ char *get_cwd (void) /*{{{*/
 	exit_error ("Unable to get cwd.", 0);
 #else
 	struct stat st1, st2;
-	int en = errno;
 	
 	if ((NULL == (c = getenv ("PWD")))
 	    || (-1 == stat (c, &st1))
@@ -138,12 +139,12 @@ char *get_cwd (void) /*{{{*/
 	    || (st1.st_dev != st2.st_dev)
 	    || (st1.st_ino != st2.st_ino))
 	  {
-	     char buf[256];
-	     
-	     sprintf (buf, "Unable to get CWD: errno = %d", en);
-	     exit_error (buf, 0);
+	     jed_vmessage (1, "Unable to get CWD.  Assuming /");
+	     if (-1 == chdir ("/"))
+	       exit_error ("Unable to change directory to /", 0);
+	     c = "/";
 	  }
-	
+
 	strncpy (cwd, c, sizeof (cwd));
 	cwd [sizeof (cwd) - 1] = 0;
 #endif
@@ -164,6 +165,8 @@ static void set_jed_root (char *pgm)
    char *jr, *jl;
    char jed_lib [JED_MAX_PATH_LEN + 1];
 
+   *jed_lib = 0;
+
 #ifdef VMS
    jr = "JED_ROOT:";
 #else
@@ -245,7 +248,7 @@ static int main_initialize (int argc, char **argv)
 #ifdef __MSDOS__
    define_word("0-9a-zA-Z\200-\232\240-\245\341-\353");
 #else
-   define_word("0-9a-zA-Z\277-\326\330-\337\340-\366\370-\377");
+   define_word("0-9a-zA-Z\300-\326\330-\366\370-\377");
 #endif
 
 #if JED_HAS_DISPLAY_TABLE
@@ -281,9 +284,16 @@ static int main_initialize (int argc, char **argv)
 	  {
 	     Batch = 2;
 	     SLang_Traceback = 1;
+	     Jed_Load_Quietly = 1;
+	  }
+	else if (!strcmp (argv[1], "-help"))
+	  {
+	     Batch = 1;
+	     SLang_Traceback = 1;
+	     Jed_Load_Quietly = 1;
 	  }
      }
-   
+
    if (Batch == 0)
      (*tt_get_terminfo) ();
 
@@ -424,7 +434,6 @@ static int main_initialize (int argc, char **argv)
    return 0;
 }
 
-   
 int main(int argc, char **argv) /*{{{*/
 {
    if (SLang_Version < SLANG_VERSION)
