@@ -63,6 +63,7 @@ typedef struct _Menu_Popup_Type
    int max_row;
    int max_col;
    unsigned int min_width;
+   int visible_node_offset;
 }
 Menu_Popup_Type;
 
@@ -911,7 +912,6 @@ static int copy_menu (Menu_Popup_Type *dest, Menu_Node_Type *src)
  *  SLsmg MenuBar interface
  */
 
-#ifndef IBMPC_SYSTEM
 static void simulate_hline (unsigned int n)
 {
    while (n--)
@@ -944,7 +944,22 @@ static void simulate_box (int r, int c, unsigned int dr, unsigned int dc)
 	SLsmg_write_nchars ("|", 1);
      }
 }
-#endif
+
+/* I would prefer to use real up/down arrows but I cannot depend upon their
+ * portability.
+ */
+static void draw_up_arrow (int r, int c)
+{
+   SLsmg_gotorc (r, c);
+   SLsmg_write_string ("(-)");
+}
+
+static void draw_down_arrow (int r, int c)
+{
+   SLsmg_gotorc (r, c);
+   SLsmg_write_string ("(+)");
+}
+
 static void draw_name (char *name, int color0, int color1, unsigned int field_width)
 {
    char *s;
@@ -1170,8 +1185,11 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
 
    if (r + (int)dr >= Jed_Num_Screen_Rows)
      {
-	if ((int)dr > Jed_Num_Screen_Rows)
-	  r = 0;
+	if ((int)dr >= Jed_Num_Screen_Rows)
+	  {
+	     r = 1;
+	     dr = Jed_Num_Screen_Rows - 1;
+	  }
 	else
 	  r = Jed_Num_Screen_Rows - (int)dr;
      }
@@ -1179,11 +1197,9 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
    SLsmg_set_color_in_region (JMENU_SHADOW_COLOR, r + 1, c + 1, dr, dc);
    SLsmg_set_color (JMENU_POPUP_COLOR);
    /* SLsmg_fill_region (r, c, dr, dc, ' '); */
-#ifndef IBMPC_SYSTEM
    if (Jed_Simulate_Graphic_Chars)
      simulate_box (r, c, dr, dc);
    else
-#endif
      SLsmg_draw_box (r, c, dr, dc);
 
    p->row = r;
@@ -1191,15 +1207,38 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
    p->max_row = r + dr;
    p->max_col = c + dc;
 
-   l = p->subnodes;
-   lmax = l + p->num_subnodes;
-   
-   c++;
-   r++;
-
    /* Make sure a selection is present */
    (void) get_selected_menu_item (p);
 
+   l = p->subnodes;
+   lmax = l + p->num_subnodes;
+   
+   if (p->num_subnodes + 2 > dr)
+     {
+	unsigned int page;
+	unsigned int nr = dr - 2;
+	unsigned int col;
+	
+	col = c + dc - 4;
+	page = p->active_node / nr;
+	if (page)
+	  {
+	     l += page * nr;
+	     draw_up_arrow (r, col);
+	  }
+	if (lmax > l + nr)
+	  {
+	     lmax = l + nr;
+	     draw_down_arrow (r + dr - 1, col);
+	  }
+	else
+	  l = lmax - nr;
+     }
+   
+   p->visible_node_offset = (int) (l - p->subnodes);
+
+   c++;
+   r++;
    p_active = NULL;
    active_row = r;
    active_col = c;
@@ -1239,11 +1278,9 @@ static int draw_popup_menu (Menu_Popup_Type *p, int r, int c)
 	  {
 	   case MENU_NODE_SEPARATOR:
 	     SLsmg_write_nchars (" ", 1);
-#ifndef IBMPC_SYSTEM
 	     if (Jed_Simulate_Graphic_Chars)
 	       simulate_hline (dc - 1);
 	     else
-#endif
 	       SLsmg_draw_hline (dc-1);
 	     SLsmg_write_nchars (" ", 1);
 	     break;
@@ -1666,7 +1703,9 @@ static SLKeymap_Function_Type Menu_Keymap_Functions [] =
 };
 
 #ifndef IBMPC_SYSTEM
+# ifdef HAS_MOUSE
 static int xterm_mouse_cmd (void);
+# endif
 #endif
 
 static int init_menu_keymap (void)
@@ -1720,7 +1759,9 @@ static int init_menu_keymap (void)
    SLkm_define_key ("^(kP)", (FVOID_STAR) pgup_menu_cmd, Menu_Keymap);
    SLkm_define_key ("^(kN)", (FVOID_STAR) pgdn_menu_cmd, Menu_Keymap);
 # endif
+# ifdef HAS_MOUSE
    SLkm_define_key ("\033[M", (FVOID_STAR) xterm_mouse_cmd, Menu_Keymap);
+# endif
 #else				       /* IBMPC_SYSTEM */
 #include "doskeys.h"
    
@@ -1798,11 +1839,16 @@ static int exec_menubar_callback (Menu_Bar_Type *b, SLang_Name_Type **ntp)
 void jed_notify_menu_buffer_changed (void)
 {
    Menu_Bar_Type *b;
-   
+   int err;
+
    if (NULL == (b = get_active_menubar ()))
      return;
 
+   err = SLang_Error;
+   SLang_Error = 0;
    (void) exec_menubar_callback (b, &b->init_callback);
+   if (SLang_Error == 0)
+     SLang_Error = err;
 }
 
 
@@ -2319,6 +2365,7 @@ int jed_init_menus (void)
    return 0;
 }
 
+#ifdef HAS_MOUSE
 static int select_menu_via_rc (int type, int r, int c)
 {
    Menu_Popup_Type *p;
@@ -2347,7 +2394,7 @@ static int select_menu_via_rc (int type, int r, int c)
 	  {
 	     if (type != JMOUSE_DRAG)
 	       return -1;
-	     
+
 	     if (Active_Popup->type != MENU_NODE_MENUBAR)
 	       return 0;
 	  }
@@ -2386,6 +2433,7 @@ static int select_menu_via_rc (int type, int r, int c)
    if (p == Active_Popup)
      {
 	r -= (p->row + 1);
+	r += p->visible_node_offset;
 	if ((r >= (int)p->num_subnodes) || (r < 0))
 	  return 0;
 	
@@ -2406,7 +2454,6 @@ static int select_menu_via_rc (int type, int r, int c)
    return 0;
 }
 
-
 int jed_menu_handle_mouse (unsigned int type, 
 			   int x, int y, int button, int shift)
 {
@@ -2446,6 +2493,6 @@ static int xterm_mouse_cmd (void)
 }
 
 
-#endif
-
+#endif 				       /* !IBMPC_SYSTEM */
+#endif				       /* HAS_MOUSE */
 #endif				       /* JED_HAS_TTY_MENUS */
