@@ -100,6 +100,12 @@ static void set_macro_state(void) /*{{{*/
 
 /*}}}*/
 
+void jed_abort_keyboard_macro (void)
+{
+   Defining_Keyboard_Macro = Executing_Keyboard_Macro = 0;
+   set_macro_state();
+}
+
 /* if not in the mini buffer and if during keyboard macro, allow user to enter
    different text each time macro is executed */
 
@@ -233,12 +239,13 @@ int jed_getkey (void) /*{{{*/
 
 void msg_error(char *msg) /*{{{*/
 {
-   Defining_Keyboard_Macro = Executing_Keyboard_Macro = 0;
-   /* flush_input(); */
    JWindow->trashed = 1;
+#if 0
+   Defining_Keyboard_Macro = Executing_Keyboard_Macro = 0;
    Read_This_Character = NULL;
    Repeat_Factor = NULL;
    set_macro_state();
+#endif
    if (Batch) fprintf(stderr, "%s\n", msg);
    
    if (!SLang_Error) SLang_Error = INTRINSIC_ERROR;
@@ -474,7 +481,7 @@ int execute_keyboard_macro() /*{{{*/
      {
 	Repeat_Factor = NULL;
 	/* ch = *Macro_Buffer_Ptr++; */
-	do_key();
+	(void) jed_do_key();
 	if (SLKeyBoard_Quit || (*Error_Buffer)) break;
      }
    Macro_Illegal_Now = 0;
