@@ -43,8 +43,6 @@
 #include "jprocess.h"
 #endif
 
-int Abort_Char = 7;		       /* scan code for G (control) */
-
 #include "dos_os2.c"
 
 extern char *get_cwd (void);
@@ -155,7 +153,7 @@ static void os2_kbdhandler_thread (void)
 	  {
 	     /* found a char process it */
 
-	     if (keyInfo.chChar == Abort_Char)
+	     if (keyInfo.chChar == Jed_Abort_Char)
 	       {
 		  if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 		  SLKeyBoard_Quit = 1;
