@@ -1,4 +1,4 @@
-/* Copyright (c) 1992, 1998 John E. Davis
+/* Copyright (c) 1992, 1998, 2000 John E. Davis
  * This file is part of JED editor library source.
  *
  * You may distribute this file under the terms the GNU General Public
@@ -73,6 +73,11 @@ static void version (void)
 {
    fprintf (stdout, "rgrep version %d.%d\n",
 	    RGREP_MAJOR_VERSION, RGREP_MINOR_VERSION);
+   fprintf (stdout, "S-Lang version: %s\n", SLang_Version_String);
+   if (SLang_Version != SLANG_VERSION)
+     fprintf (stdout, "*** Compiled against S-Lang %d but linked to %d\n",
+	      SLANG_VERSION, SLang_Version);
+
    exit (0);
 }
 
@@ -573,15 +578,16 @@ static int unix_is_dir (char *dir, int force_link)
 #endif
    struct stat buf;
    int mode;
-   
+
+   if (stat(dir, &buf)) return -1;
+   if ((buf.st_mode & S_IFMT) == S_IFREG)
+     return 0;
+
 #ifdef S_IFLNK
-   if (Follow_Links || force_link)
-     {
-#endif
-	if (stat(dir, &buf)) return -1;
-#ifdef S_IFLNK
-     }
-   else if (lstat(dir, &buf) < 0) return -1;
+   if ((0 == Follow_Links)
+       && (0 == force_link)
+       && (-1 == lstat(dir, &buf)))
+     return -1;
 #endif
    
    mode = buf.st_mode & S_IFMT;
