@@ -43,9 +43,7 @@
 volatile int Jed_Resize_Pending;
 char *MiniBuf_Get_Response_String;
 
-#ifndef IBMPC_SYSTEM
 int Jed_Simulate_Graphic_Chars;
-#endif
 
 typedef struct Screen_Type
 {
@@ -66,7 +64,7 @@ int Cursor_Motion;    /* indicates cursor movement only -1 ^ v +1 < > */
 int Jed_Dollar = '$';
 int User_Prefers_Line_Numbers = 0;
 int Mode_Has_Syntax_Highlight;
-int Wants_Syntax_Highlight;	       /* if non-zero, highlight the syntax.
+int Wants_Syntax_Highlight = 1;	       /* if non-zero, highlight the syntax.
 					*/
 int Wants_Attributes = 1;
 int Wants_HScroll = 20;		       /* controls automatic horizontal
@@ -1049,7 +1047,7 @@ static int update_1(Line *top, int force)
 {
    int i;
    Window_Type *w;
-   int did_eob = 0, time_has_expired = 0;
+   int did_eob, time_has_expired = 0;
    
    if (Batch ||
        (!force
