@@ -250,7 +250,9 @@ void write_syntax_highlight (Line *l, unsigned int len)
    if (st == NULL) return;
 
 #if JED_HAS_DFA_SYNTAX
-   if ((st->hilite != NULL) && (st->hilite->dfa != NULL))
+   if (st->use_dfa_syntax
+       && (st->hilite != NULL) 
+       && (st->hilite->dfa != NULL))
      {
 	dfa_syntax_highlight (p, pmax, st);
 	return;
@@ -476,10 +478,9 @@ void init_syntax_highlight (void) /*{{{*/
     }
 
 #if JED_HAS_DFA_SYNTAX
-   if ((NULL != st->hilite) && (NULL != st->hilite->dfa))
-     {
-	return;
-     }
+   if (st->use_dfa_syntax
+       && ((NULL != st->hilite) && (NULL != st->hilite->dfa)))
+     return;
 #endif
    
    Char_Syntax = st->char_syntax;
