@@ -49,14 +49,13 @@ extern int sys_findnext(char *);
 extern unsigned long  sys_time(void);
 extern int Meta_Char;
 extern int DEC_8Bit_Hack;
-extern int Abort_Char;
 extern void map_character(int *, int *);
 extern int make_random_number(int *, int *);
 extern int ch_dir(char *);
 extern void deslash(char *);
 extern char *slash2slash (char *);
 
-#if defined(__MSDOS_16BIT__) || defined(__os2__)
+#if defined(__MSDOS_16BIT__) || defined(__os2__) || defined(__WIN32__)
 extern int NumLock_Is_Gold;
 #endif
 
@@ -64,7 +63,7 @@ extern int NumLock_Is_Gold;
 extern int IsHPFSFileSystem(char *);
 #endif
 
-#if defined(__GO32__) || defined(__WATCOMC__)
+#if defined(__GO32__) || (defined(__WATCOMC__) && !defined(__QNX__))
 extern void i386_check_kbd(void);
 extern int i386_access (char *, int);
 #endif
