@@ -125,13 +125,9 @@ void wrap_line(int format) /*{{{*/
 
 /*}}}*/
 
-/* Here a paragraph follows either an indentation, a '\\' char or two
-   '\n' chars.  or a '%' char since tex uses this */
 static int is_paragraph_sep(void) /*{{{*/
 {
-   unsigned char *p;
    int ret;
-   p = CLine->data;
    
    if (CBuf->par_sep != NULL)
      {
@@ -144,8 +140,17 @@ static int is_paragraph_sep(void) /*{{{*/
 	  }
 	return ret;
      }
-   if ((*p == '\n') || (*p == '\\') || (*p == '%')) return(1);
-   return(0);
+   
+   push_spot ();
+   (void) bol ();
+   skip_whitespace ();
+   if (eolp ())
+     {
+	pop_spot ();
+	return 1;
+     }
+   pop_spot ();
+   return 0;
 }
 
 /*}}}*/
@@ -153,22 +158,17 @@ static int is_paragraph_sep(void) /*{{{*/
 int backward_paragraph(void) /*{{{*/
 {
    Line *prev = CLine->prev;
-   int ro = CBuf->flags & READ_ONLY;
    
    if (NULL == CBuf->par_sep)
-     {
-	CBuf->par_sep = SLang_get_function("is_paragraph_separator");
-     }
+     CBuf->par_sep = SLang_get_function("is_paragraph_separator");
 
    Point = 0;
    if (prev == NULL) return(0);
    CLine = prev; LineNum--;
-   
-   while(1)
+
+   while ((0 == is_paragraph_sep ())
+	  && (CLine->prev != NULL))
      {
-	eol();
-	if (!ro) trim_whitespace();
-	if (is_paragraph_sep() || (CLine->prev == NULL)) break;
 	CLine = CLine->prev; LineNum--;
      }
    Point = 0;
@@ -179,20 +179,14 @@ int backward_paragraph(void) /*{{{*/
 
 int forward_paragraph(void) /*{{{*/
 {
-   int ro = CBuf->flags & READ_ONLY;   
-   
    if (NULL == CBuf->par_sep)
-     {
-	CBuf->par_sep = SLang_get_function("is_paragraph_separator");
-     }
+     CBuf->par_sep = SLang_get_function("is_paragraph_separator");
 
-   while(1)
+   while (NULL != CLine->next)
      {
-	if (CLine->next == NULL) break;
-	CLine = CLine->next;  LineNum++;
-	eol();
-	if (!ro) trim_whitespace();
-	if (is_paragraph_sep()) break;
+	CLine = CLine->next; LineNum++;
+	if (is_paragraph_sep()) 
+	  break;
      }
 
    eol();
@@ -392,7 +386,7 @@ void transform_region(int *what) /*{{{*/
    line = CLine;
    pop_mark(&Number_One);
 
-   p = CLine->data + Point; 
+   /* p = CLine->data + Point; */ 
    while (1)
      {
 	if (line == CLine) n = pnt; else n = CLine->len;
@@ -417,7 +411,7 @@ void transform_region(int *what) /*{{{*/
 	       goto the_return;
 
 	     Point++;
-	     p = CLine->data + Point;
+	     /* p = CLine->data + Point; */
 	     /* drop through */
 	     
 	   case 'd': 
@@ -447,7 +441,7 @@ void transform_region(int *what) /*{{{*/
 	CLine = CLine->next;
 	LineNum++;
 	Point = 0;
-	p = CLine->data;
+	/* p = CLine->data; */
      }
    the_return:
    pop_spot();
