@@ -13,6 +13,10 @@
 #include <stdio.h>
 #include "sysdep.h"
 
+#ifdef HAVE_GLOB_H
+# include <glob.h>
+#endif
+
 #if JED_HAS_SUBPROCESSES
 # include "jprocess.h"
 #endif
@@ -36,7 +40,7 @@
 #endif
 
 #ifdef __QNX__
-# include <termios.h>
+/* # include <termios.h> */
 # include <sys/select.h>
 #endif
 
@@ -85,9 +89,7 @@
 static int Read_FD = TTY_DESC;
 static int Max_Fd = TTY_DESC;
 
-int Flow_Control;
-int Abort_Char = 7;		       /* scan code for G (control) */
-
+static int Flow_Control;
 
 #ifndef REALLY_HAVE_TERMIOS_H
 typedef struct /*{{{*/
@@ -106,11 +108,11 @@ static TTY_Termio_Type Old_TTY;
 
 #ifdef REALLY_HAVE_TERMIOS_H
 # define GET_TERMIOS(fd, x) tcgetattr(fd, x)
-#if 0
-# define SET_TERMIOS(fd, x) tcsetattr(fd, TCSAFLUSH, x)
-#else
-# define SET_TERMIOS(fd, x) tcsetattr(fd, TCSADRAIN, x)
-#endif
+# if 0
+#  define SET_TERMIOS(fd, x) tcsetattr(fd, TCSAFLUSH, x)
+# else
+#  define SET_TERMIOS(fd, x) tcsetattr(fd, TCSADRAIN, x)
+# endif
 #else
 # ifdef TCGETS
 #  define GET_TERMIOS(fd, x) ioctl(fd, TCGETS, x)
@@ -171,11 +173,22 @@ static void handle_interrupt (void)
 {
    if (Updating_Screen)
      return;
-   
+
    while (Jed_Resize_Pending)
      jed_resize_display ();
 }
 
+static int unix_set_abort_char (unsigned char ch)
+{
+   if (Jed_Abort_Char == ch)
+     return 0;
+
+   reset_tty ();
+   Jed_Abort_Char = ch;
+   init_tty ();
+   return 0;
+}
+
 int init_tty (void) /*{{{*/
 {
    TTY_Termio_Type newtty;
@@ -203,13 +216,13 @@ int init_tty (void) /*{{{*/
    tt_enable_cursor_keys();
    while (-1 == GET_TERMIOS(Read_FD, &Old_TTY))
      {
-	if (errno != EINTR) 
+	if (errno != EINTR)
 	  return -1;
      }
-   
+
    while (-1 == GET_TERMIOS(Read_FD, &newtty))
      {
-	if (errno != EINTR) 
+	if (errno != EINTR)
 	  return -1;
      }
 
@@ -220,7 +233,7 @@ int init_tty (void) /*{{{*/
    newtty.s.sg_flags &= ~(CRMOD);
 /*   if (Flow_Control == 0) newtty.s.sg_flags &= ~IXON; */
    newtty.t.t_eofc = 1;
-   newtty.t.t_intrc = Abort_Char;	/* ^G */
+   newtty.t.t_intrc = Jed_Abort_Char;	/* ^G */
    newtty.t.t_quitc = 255;
    newtty.lt.t_suspc = 255;   /* to ignore ^Z */
    newtty.lt.t_dsuspc = 255;    /* to ignore ^Y */
@@ -229,22 +242,22 @@ int init_tty (void) /*{{{*/
    bskey[0] = (char) newtty.s.sg_erase;
    _Jed_Backspace_Key = bskey;
 #else
-   
-#if NEED_BAUDRATES
+
+# if NEED_BAUDRATES
    /* No need for this anymore since SLsmg is used */
 
    if (*tt_Baud_Rate == 0)
      {
-/* Note:  if this generates an compiler error, simply remove 
+/* Note:  if this generates an compiler error, simply remove
    the statement */
-# ifdef HAVE_CFGETOSPEED
+#  ifdef HAVE_CFGETOSPEED
 	*tt_Baud_Rate = cfgetospeed (&newtty);
-# endif
-	
+#  endif
+
 	*tt_Baud_Rate = (*tt_Baud_Rate > 0) && (*tt_Baud_Rate < 19) ?
 	  Baud_Rates[*tt_Baud_Rate] : -1;
      }
-#endif
+# endif
 
    newtty.c_iflag &= ~(ECHO | INLCR | ICRNL);
 # ifdef ISTRIP
@@ -258,8 +271,8 @@ int init_tty (void) /*{{{*/
    newtty.c_cc[VTIME] = 0;
    newtty.c_cc[VEOF] = 1;
    newtty.c_lflag = ISIG | NOFLSH;
-   
-   newtty.c_cc[VINTR] = Abort_Char;   /* ^G */
+
+   newtty.c_cc[VINTR] = Jed_Abort_Char;   /* ^G */
    newtty.c_cc[VQUIT] = NULL_VALUE;
    newtty.c_cc[VSUSP] = NULL_VALUE;   /* to ignore ^Z */
 
@@ -278,7 +291,7 @@ int init_tty (void) /*{{{*/
 	if (errno != EINTR)
 	  return -1;
      }
-   
+
 #ifdef HAS_MOUSE
    if ((X_Open_Mouse_Hook != NULL)
        && ((JMouse_Fd = (*X_Open_Mouse_Hook)()) >= 0))
@@ -286,10 +299,10 @@ int init_tty (void) /*{{{*/
 	if (JMouse_Fd > Read_FD) Max_Fd = JMouse_Fd;
      }
 #endif
-   
+
    X_Update_Open_Hook = unix_update_open;
    X_Update_Close_Hook = unix_update_close;
-
+   X_Set_Abort_Char_Hook = unix_set_abort_char;
    return 0;
 }
 
@@ -304,14 +317,14 @@ void reset_tty (void) /*{{{*/
 	if (X_Reset_Term_Hook != NULL) (*X_Reset_Term_Hook) ();
 	return;
      }
-   
+
    while (-1 == SET_TERMIOS(Read_FD, &Old_TTY))
      {
-	if (errno != EINTR) 
+	if (errno != EINTR)
 	  break;
      }
-   
-   /* This statement ensures init_tty will not try to change output_rate 
+
+   /* This statement ensures init_tty will not try to change output_rate
       (when coming back from suspension) */
 #if NEED_BAUDRATES
    if (*tt_Baud_Rate == 0) *tt_Baud_Rate = -1;
@@ -334,20 +347,20 @@ unsigned char sys_getkey (void) /*{{{*/
 #else
    int all = 0;
 #endif
-   
+
    while (1)
      {
 	handle_interrupt ();
-	
+
 	if (SLKeyBoard_Quit) break;
-	
+
 	/* sleep for 45 second and try again */
 	if (sys_input_pending(&n, all) > 0)
 	  break;
-	
+
 	if (SLKeyBoard_Quit)
 	  break;
-	
+
 	/* update status line in case user is displaying time */
 	if (Display_Time || all)
 	  {
@@ -355,33 +368,33 @@ unsigned char sys_getkey (void) /*{{{*/
 	     update((Line *) NULL, 0, 1, 0);
 	  }
      }
-   
-   if (SLKeyBoard_Quit) 
+
+   if (SLKeyBoard_Quit)
      {
 	SLKeyBoard_Quit = 0;
-	return Abort_Char;
+	return Jed_Abort_Char;
      }
-   
+
    if (X_Read_Hook != NULL) return (X_Read_Hook ());
-   
+
    /* Something may have stuffed it, e.g., mouse */
-   
+
    if (Input_Buffer_Len) return my_getkey ();
-   
+
 #ifdef HAS_MOUSE
    if (JMouse_Hide_Mouse_Hook != NULL) (*JMouse_Hide_Mouse_Hook) (0);
 #endif
-   
+
    while (1)
      {
 	int nread;
-	
+
 	if (SLKeyBoard_Quit)
 	  {
 	     SLKeyBoard_Quit = 0;
-	     return Abort_Char;
+	     return Jed_Abort_Char;
 	  }
-	
+
 	if (-1 == (nread = read (Read_FD, (char *) &c, 1)))
 	  {
 #ifdef EINTR
@@ -396,7 +409,7 @@ unsigned char sys_getkey (void) /*{{{*/
 
 	if (nread > 0)
 	  return c;
-	
+
 	if (nread == 0)
 	  exit_error ("getkey: EOF seen!", 0);
      }
@@ -423,10 +436,10 @@ static int beos_tty_select(int fd, fd_set *mask, struct timeval *wait)
 
    if (wait == NULL)
      usecs = 1000000LL * 60 * 60 * 24 * 365;
-   else 
+   else
      usecs = wait->tv_usec + wait->tv_sec * 1000000LL;
 
-   while (usecs >= 0) 
+   while (usecs >= 0)
      {
 	if (wait == NULL)
 	  howmany = 1;
@@ -434,7 +447,7 @@ static int beos_tty_select(int fd, fd_set *mask, struct timeval *wait)
 	  howmany = 0;
 
 	ret = ioctl(fd, 'ichr', &howmany);
-	if (ret < 0) 
+	if (ret < 0)
 	  {
 	     FD_CLR(fd, mask);
 	     return -1;
@@ -442,11 +455,11 @@ static int beos_tty_select(int fd, fd_set *mask, struct timeval *wait)
 
 	if ((ret >= 0) && (howmany > 0))
 	  return 1;
-	
+
 	snooze(1000LL);
 	usecs -= 1000;
      }
-   
+
    FD_CLR(fd, mask);
    return 0;
 }
@@ -464,19 +477,19 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
    int i;
 #endif
    static int bad_select;
-   
+
    if ((all >= 0) && (Input_Buffer_Len || Batch)) return(Input_Buffer_Len);
-   
+
    if (bad_select) return -1;
-   
+
    top:
-   
+
    FD_ZERO(&Read_FD_Set);
    secs = *tsecs / 10;
    usecs = (*tsecs % 10) * 100000;
    wait.tv_sec = secs;
    wait.tv_usec = usecs;
-   
+
    if (all >= 0)
      {
 	if ((X_Input_Pending_Hook != NULL)
@@ -493,7 +506,7 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 	maxfd = Max_Fd;
      }
    else maxfd = -1;
-   
+
 #if JED_HAS_SUBPROCESSES
    if (all)
      {
@@ -513,8 +526,8 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
    ret = select(maxfd + 1, &Read_FD_Set, NULL, NULL, &wait);
 #endif
 
-   if (ret == 0) return 0; 
-   
+   if (ret == 0) return 0;
+
    if ((ret == -1) && (errno == EINTR))
      handle_interrupt ();
 
@@ -525,17 +538,17 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 	return -1;
      }
 #endif
-   
+
 #ifdef EBADF
    if ((ret < 0) && (errno == EBADF))
      {
-	/* Oh Geez.  We have an invalid file descriptor.  Well, there are 
+	/* Oh Geez.  We have an invalid file descriptor.  Well, there are
 	 * several options.  Here is what I am choosing: If we are selecting
 	 * on subprocesses and the bad descriptor appears to be the subprocess
 	 * then ignore select on suprocesses.  Hopefully the subprocess child
 	 * handlers will fire and illiminate this situation.  If it is the
 	 * mouse, then turn off the mouse.  If it is the keyboard, then exit.
-	 */	
+	 */
 	FD_ZERO (&Read_FD_Set);
 	if (all >= 0)
 	  {
@@ -559,14 +572,14 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 	goto top;
      }
 #endif
-   
+
    if (all >= 0)
      {
 	if ((X_Input_Pending_Hook != NULL) && (ret > 0)
 	    && (FD_ISSET(Read_FD, &Read_FD_Set)))
 	  {
 	     if ((*X_Input_Pending_Hook) ()) return 1;
-	     
+
 	     /* Nothing there so try to time out again --- too bad select does
 	      * not inform of of how far we got.
 	      */
@@ -579,11 +592,11 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 	     return (*X_Input_Pending_Hook) ();
 #endif
 	  }
-	
+
 	if (ret < 0) return ret;
-	
+
 	if (FD_ISSET(Read_FD, &Read_FD_Set)) return ret;
-	
+
 	/* Nothing to read from the terminal so while we are waiting
 	 * for tty input, read from those that are available.  TTY input
 	 * always gets preference.
@@ -602,9 +615,9 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 #endif
      }
    /* all >= 0 */
-   
+
    if (ret <= 0) return ret;
-   
+
 #if JED_HAS_SUBPROCESSES
    i = 0;
    while (i < Num_Subprocesses)
@@ -627,7 +640,7 @@ void sys_pause (int ms) /*{{{*/
 
    secs = ms / 1000;
    usecs = (ms % 1000) * 1000;
-   
+
    wait.tv_sec = secs;
    wait.tv_usec = usecs;
 
@@ -636,7 +649,6 @@ void sys_pause (int ms) /*{{{*/
 
 /*}}}*/
 
-
 /* returns 0 on failure, 1 on sucess */
 int sys_delete_file(char *filename) /*{{{*/
 {
@@ -663,43 +675,41 @@ void sys_suspend (void) /*{{{*/
 
 /*}}}*/
 
-
-
 /* returns 0 if file does not exist, 1 if it is not a dir, 2 if it is */
 int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 {
    struct stat buf;
    int m;
-   
+
    if (what)
      {
 	if (-1 == chmod(file, *mode))
 	  goto return_error;
-	
+
 	if (-1 == chown(file, (uid_t) *uid, (uid_t) *gid))
 	  {
 	     if (-1 == stat (file, &buf))
 	       goto return_error;
-	     
+
 	     (void) chown(file, (uid_t) buf.st_uid, (uid_t) *gid);
 	  }
 	return 0;
      }
-   
+
    if (-1 == stat(file, &buf))
      goto return_error;
-   
+
    m = buf.st_mode;
    *uid = buf.st_uid;
    *gid = buf.st_gid;
-   
+
 /* AIX requires this */
 #ifdef _S_IFDIR
 # ifndef S_IFDIR
 #  define S_IFDIR _S_IFDIR
 # endif
 #endif
-   
+
 #ifndef S_ISDIR
 # ifdef S_IFDIR
 #  define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
@@ -707,14 +717,14 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 #  define S_ISDIR(m) 0
 # endif
 #endif
-   
+
    *mode = m & 0777;
-   
+
    if (S_ISDIR(m)) return (2);
    return(1);
-   
+
    return_error:
-   
+
    switch (errno)
      {
       case EACCES: return(-1); /* es = "Access denied."; break; */
@@ -726,32 +736,83 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 
 /*}}}*/
 
+#ifdef HAVE_GLOB_H
+static glob_t Globbuf;
+static int Glob_Count = -1;
 
-/* use berkeley interface
- * #include <sys/dir.h>
-*/
+int sys_findnext(char *file) /*{{{*/
+{
+   while ((unsigned int) Glob_Count < (unsigned int) Globbuf.gl_pathc)
+     {
+	char *f = Globbuf.gl_pathv[Glob_Count++];
+	if (strlen (f) < JED_MAX_PATH_LEN)
+	  {
+	     strcpy (file, f);
+	     return 1;
+	  }
+     }
+
+   Glob_Count = -1;
+   globfree(&Globbuf);
+   return 0;
+}
+
+/*}}}*/
+int sys_findfirst(char *the_file) /*{{{*/
+{
+   char *f, the_path[JED_MAX_PATH_LEN+2], *file;
+   unsigned int len;
+
+   file = expand_filename(the_file);
+   f = extract_file(file);
+   len = strlen (file);
+   if (len >= JED_MAX_PATH_LEN)
+     {
+	jed_verror ("path name is too long");
+	return 0;
+     }
+   strcpy (the_path, file);
+
+   if (NULL == strchr(f, '*'))
+     strcat (the_path, "*");
+
+   if (Glob_Count != -1)
+     {
+	Glob_Count = -1;
+	globfree(&Globbuf);
+     }
+
+   if (glob(the_path, GLOB_MARK, NULL, &Globbuf) != 0) return 0;
+
+   Glob_Count = 0;
+   return sys_findnext (the_file);
+}
+
+/*}}}*/
+
+#else
 
 static char Found_Dir[JED_MAX_PATH_LEN];
 static char Found_File[JED_MAX_PATH_LEN];
 static int File_Len;
 
-#if HAVE_DIRENT_H
-# include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
-#else
-# define dirent direct
-# define NAMLEN(dirent) (dirent)->d_namlen
-# define NEED_D_NAMLEN
-# if HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
+# if HAVE_DIRENT_H
+#  include <dirent.h>
+#  define NAMLEN(dirent) strlen((dirent)->d_name)
+# else
+#  define dirent direct
+#  define NAMLEN(dirent) (dirent)->d_namlen
+#  define NEED_D_NAMLEN
+#  if HAVE_SYS_NDIR_H
+#   include <sys/ndir.h>
+#  endif
+#  if HAVE_SYS_DIR_H
+#   include <sys/dir.h>
+#  endif
+#  if HAVE_NDIR_H
+#   include <ndir.h>
+#  endif
 # endif
-# if HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#  include <ndir.h>
-# endif
-#endif
 
 static DIR *Dirp;
 
@@ -759,27 +820,27 @@ static DIR *Dirp;
  * versions do.  Unfortunately, it is a sad fact of life that Unix does NOT
  * support wildcards.  Strangely enough, most Unix users are totally unaware
  * of this fact.
- * 
+ *
  * To add wild card support, I need to use the regular expression package.
- * Specifically, in sys_findfirst, I would need to scan the file spec for 
+ * Specifically, in sys_findfirst, I would need to scan the file spec for
  * wild cards making the replacements: . --> \., ? -> ., and * -> .* in that
  * order.  I should also quote the special characters ($, etc...) but I will
  * not worry about this.  Finally, I will have to prefix it with '^' since
  * the match will start at the beginning of the string.
- * 
+ *
  *    Remark: to implement something like a real wildcard file renaming routine
- *    would require even more processing.  Consider something like: 
+ *    would require even more processing.  Consider something like:
  *       rename *.c~ *.bak
  *    This mean that the first expression (*.c~) would have to be converted to:
  *      \(.*\)\.c~
  *    and the second (*.bak) would be determined to be: \1.bak  where \1 is
- *    the expression matched by the first wildcard.  
- * 
+ *    the expression matched by the first wildcard.
+ *
  * After converting the wildcard file name to a regexp file name, I would then
  * have to compile it and save the compiled expression for use in sys_findnext.
- * There, I would use the regexp string matching routine instead of the 
+ * There, I would use the regexp string matching routine instead of the
  * strcmp which is now used.
- * 
+ *
  * Mainly, the findfirst/findnext are used by the completion routines.  This
  * means that there is an implicit '.*' attached to the end of the filespec.
  * This will have to be dealt with.
@@ -787,20 +848,20 @@ static DIR *Dirp;
 int sys_findnext(char *file) /*{{{*/
 {
    struct dirent *dp;
-   
+
    if (Dirp == NULL)
      return 0;
 
    while (1)
      {
-	if (NULL == (dp = readdir(Dirp))) 
+	if (NULL == (dp = readdir(Dirp)))
 	  {
 	     closedir (Dirp); Dirp = NULL;
 	     return 0;
 	  }
-#ifdef NEED_D_NAMLEN
+# ifdef NEED_D_NAMLEN
 	dp->d_name[dp->d_namlen] = 0;
-#endif
+# endif
 	if (!strncmp(Found_File, dp->d_name, File_Len)) break;
      }
    strcpy(file, Found_Dir); strcat(file, dp->d_name);
@@ -812,21 +873,36 @@ int sys_findnext(char *file) /*{{{*/
 int sys_findfirst(char *the_file) /*{{{*/
 {
    char *f, *file;
-   
+
    file = expand_filename(the_file);
    f = extract_file(file);
-   
+
    strcpy (Found_Dir, file);
    strcpy (Found_File, f);
    File_Len = strlen(f);
-   
+
    Found_Dir[(int) (f - file)] = 0;
-   
+
    if (Dirp != NULL) closedir(Dirp);
-   
+
    if (NULL == (Dirp = (DIR *) opendir(Found_Dir))) return(0);
    strcpy(the_file, file);
    return sys_findnext(the_file);
 }
 
 /*}}}*/
+
+#endif				       /* GLOB_H */
+
+/* if non-zero, Flow control is enabled */
+void enable_flow_control (int *mode) /*{{{*/
+{
+   /* This kills X windows.  For the time being, work around it as follows */
+   if (X_Init_Term_Hook != NULL) return;
+   Flow_Control = *mode;
+   reset_tty();
+   init_tty();
+}
+
+/*}}}*/
+
