@@ -51,10 +51,8 @@ typedef struct {                /* Terminal characteristics   */
 
 TermChar_Type Old_Term_Char, New_Term_Char;
 
-int Abort_Char = 7;		       /* scan code for G (control) */
-
 /* This serves to identify the channel we are reading input from.  */
-short This_Term;
+static short This_Term;
 
 
 typedef struct /*{{{*/
@@ -150,7 +148,7 @@ static int getkey_ast(int not_used) /*{{{*/
 
    if (c <= 255)
      {
-	if (c == Abort_Char)
+	if (c == (unsigned int) Jed_Abort_Char)
 	  {
 	     if (Ignore_User_Abort == 0) SLang_Error = 2;
 	     SLKeyBoard_Quit = 1;
@@ -381,7 +379,7 @@ unsigned char sys_getkey() /*{{{*/
    unsigned char c;
    int tsecs;
 
-   if (SLKeyBoard_Quit) return((int) Abort_Char);
+   if (SLKeyBoard_Quit) return(Jed_Abort_Char);
    
    /* Under DECWIndows, I do not know how to timeout so this will have to do
       for now */
