@@ -52,8 +52,8 @@
 #define KEY_SHIFT         1
 #define KEY_CONTROL       2
 #define KEY_ALT	 	  4
+#define KEY_ALT_GR 	  8
 
-int Abort_Char = 7;
 int NumLock_Is_Gold = 0;
 int PC_Alt_Char = 27;
 int PC_Fn_Char = 0;
@@ -98,7 +98,8 @@ static void process_key_event(KEY_EVENT_RECORD *key)
    DWORD d = key->dwControlKeyState;
 
    if (!key->bKeyDown) return;
-   if (d & (LEFT_ALT_PRESSED | RIGHT_ALT_PRESSED)) key_state |= KEY_ALT;
+   if (d & LEFT_ALT_PRESSED) key_state |= KEY_ALT;
+   if (d & RIGHT_ALT_PRESSED) key_state |= KEY_ALT_GR;
    if (d & (LEFT_CTRL_PRESSED | RIGHT_CTRL_PRESSED)) key_state |= KEY_CONTROL;
    if (d & SHIFT_PRESSED) key_state |= KEY_SHIFT;
    
@@ -108,6 +109,15 @@ static void process_key_event(KEY_EVENT_RECORD *key)
 
    switch (scan)
      {
+      case 0x00F:		       /* TAB */
+	if (key_state & KEY_SHIFT)
+	  {
+	     /* Same as DOS */
+	     buffer_keystring ("\000\017", 2);
+	     return;
+	  }
+	break;
+
       case 0x00E:		       /* backspace */
 	_putkey (127);
 	return;
@@ -147,6 +157,19 @@ static void process_key_event(KEY_EVENT_RECORD *key)
 	  }
 	break;
 
+	/* PF1 and PF2 codes */
+      case 0x0045:		       /* NUMLOCK */
+      case 0x0145:
+	if (NumLock_Is_Gold)
+	  {
+	     buffer_keystring ("\000;", 2);
+	     return;
+	  }
+	break;
+
+      case 0x0135:
+	c1 = 'Q'; break ;
+
       case 0xE02F:
 	c1 = 'Q'; break;	       /* KEYPAD SLASH */
 
@@ -278,14 +301,30 @@ static void process_key_event(KEY_EVENT_RECORD *key)
 #endif
    if (c1 != 0)
      {
-	if (c1 == Abort_Char)
+	if (c1 == (char) Jed_Abort_Char)
 	  {
 	     if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 	     SLKeyBoard_Quit = 1;
 	  }
 	if (key_state & KEY_ALT) _putkey(PC_Alt_Char);
 	_putkey(c1);
+	return;
      }
+#if 0
+   if (1)
+     {
+	static FILE *fp;
+	
+	if (fp == NULL)
+	  fp = fopen ("key.log", "w");
+	
+	if (fp != NULL)
+	  {
+	     fprintf (fp, "Key: scan=0x%X\n", scan);
+	     fflush (fp);
+	  }
+     }
+#endif
 }
 
 
@@ -574,7 +613,7 @@ unsigned char sys_getkey (void)
      {
 	SLKeyBoard_Quit = 0;
 	flush_input ();
-	return (unsigned char)Abort_Char;
+	return Jed_Abort_Char;
      }
 
    return (unsigned char) my_getkey();
