@@ -43,17 +43,23 @@
 # include "jprocess.h"
 #endif
 
+#ifdef __WIN32__
+# define HAS_WHEEL_MOUSE_SUPPORT	1
+#else
+# define HAS_WHEEL_MOUSE_SUPPORT	0
+#endif
+
 #define MSW_STRING_TYPE   1
 #define MSW_INT_TYPE      2
 #define MSW_COLOR_TYPE    3
 
+#define MAX_STRING_SIZE	  128
 #define MAX_KEYS          128
 
 #define KEY_SHIFT         1
 #define KEY_CONTROL       2
 #define KEY_ALT	 	  4
 
-int Abort_Char = 7;
 int NumLock_Is_Gold = 0;
 int PC_Alt_Char = 27;
 int PC_Fn_Char = 0;
@@ -74,6 +80,44 @@ HINSTANCE _hPrev;
 extern HINSTANCE _hPrev;
 #endif
 
+/* #define CURRENT_BG_COLOR JColors[JNORMAL_COLOR].hbrBG */
+#define CURRENT_BG_COLOR This_Window.current_color->hbrBG
+
+#if HAS_WHEEL_MOUSE_SUPPORT
+
+#ifndef WM_MOUSEWHEEL
+# define WM_MOUSEWHEEL (WM_MOUSELAST+1)  /* message that will be supported by the OS */
+#endif
+
+/* Details of the mystery mouse wheel window */
+
+#define MOUSEZ_CLASSNAME  "MouseZ"            /* wheel window class */
+#define MOUSEZ_TITLE      "Magellan MSWHEEL"  /* wheel window title */
+
+#define MSH_WHEELMODULE_CLASS (MOUSEZ_CLASSNAME)
+#define MSH_WHEELMODULE_TITLE (MOUSEZ_TITLE)
+#define MSH_MOUSEWHEEL     "MSWHEEL_ROLLMSG"
+#define MSH_WHEELSUPPORT   "MSH_WHEELSUPPORT_MSG"  /* name of msg to send to query for wheel support */
+#define MSH_SCROLL_LINES   "MSH_SCROLL_LINES_MSG"
+
+/* Details of the mouse wheel functionality - not all currently used by Wjed */
+
+typedef struct
+{
+   HWND hwnd;                 
+   UINT uiMshMsgMouseWheel,   
+        uiMshMsg3DSupport,    
+        uiMshMsgScrollLines;  
+   INT  iScrollLines;
+   BOOL fActive;
+}
+MS_Wheel_Type;
+
+static MS_Wheel_Type Wheel_Mouse;
+
+#endif                                 /* HAS_WHEEL_MOUSE_SUPPORT */
+
+
 typedef struct
 {
    char  *name;
@@ -100,12 +144,12 @@ Color_Value_Type;
 
 typedef struct
 {
-   char        title[30];
+   char        title[MAX_STRING_SIZE];
    HWND        w;
    HDC         hdc;
    int         ndc;
 
-   char        font_name[50];
+   char        font_name[MAX_STRING_SIZE];
    int         font_height;
    int         font_bold;
    int         font_width;
@@ -193,7 +237,7 @@ static LPLOGPALETTE The_Lplgpl = NULL;
 static HPALETTE The_Hpalette = NULL;
 #endif
 
-Msw_Ini_Type Msw_Ini_List[] =
+static Msw_Ini_Type Msw_Ini_List[] =
 {
      { "Font",         MSW_STRING_TYPE,   "fixed",         This_Window.font_name },
      { "FontHeight",   MSW_INT_TYPE,      "12",            &This_Window.font_height },
@@ -454,14 +498,15 @@ static COLORREF msw_get_color(char *s, char *dflt)
 
 static void msw_get_default(Msw_Ini_Type *t)
 {
-   char s[50];
+   char s[MAX_STRING_SIZE];
 
    GetProfileString(szJedSection, t->name, t->dflt, s, sizeof(s));
 
    switch (t->type)
      {
       case MSW_STRING_TYPE:
-	strcpy((char *)t->buf, s);
+	strncpy((char *)t->buf, s, MAX_STRING_SIZE);
+	((char *)t->buf)[MAX_STRING_SIZE-1] = 0;
 	break;
 
       case MSW_INT_TYPE:
@@ -478,7 +523,7 @@ static void msw_get_default(Msw_Ini_Type *t)
 static void msw_get_defaults(void)
 {
    int i = 0;
-   char buf[20];
+   char buf[128];
 
    /* Check for standard color names in INI file.
     * Add they if cannot find. */
@@ -676,12 +721,46 @@ static void copy_rect(int x1, int y1, int x2, int y2, int x3, int y3)
 
 static void blank_rect(int x1, int y1, int x2, int y2)
 {
+   char blanks[256];
+   
+   memset (blanks, ' ', sizeof (blanks));
+
+   if (This_Window.cursor_showing) hide_cursor();
+
+   get_dc ();
+   SelectObject(This_Window.hdc, This_Window.font);
+   SetTextColor(This_Window.hdc, This_Window.current_color->fg);
+   SetBkColor(This_Window.hdc, This_Window.current_color->bg);
+
+   while (y1 < y2)
+     {
+	int n = x2 - x1;
+	int xx = x1;
+
+	while (n > 0)
+	  {
+	     int dn;
+	     if (n > (int) sizeof (blanks))
+	       dn = sizeof (blanks);
+	     else
+	       dn = n;
+
+	     (void) TextOut(This_Window.hdc, xx * This_Window.font_width, 
+			    y1 * This_Window.font_height, blanks, dn);
+	     n -= dn;
+	     xx += dn;
+	  }
+	y1++;
+     }
+   release_dc ();
+#if 0
    RECT rc;
 
    SetRect(&rc, x1 * This_Window.font_width, y1 * This_Window.font_height, x2 * This_Window.font_width, y2 * This_Window.font_height);
    get_dc();
-   FillRect(This_Window.hdc, &rc, JColors[JNORMAL_COLOR].hbrBG);
+   FillRect(This_Window.hdc, &rc, CURRENT_BG_COLOR);
    release_dc();
+#endif
 }
 
 /* This routine assumes that cursor is in the correct location.  The
@@ -947,8 +1026,12 @@ static void msw_cls(void)
 
    get_dc();
    GetClientRect(This_Window.w, &rc);
-   FillRect(This_Window.hdc, &rc, JColors[JNORMAL_COLOR].hbrBG);
+   FillRect(This_Window.hdc, &rc, CURRENT_BG_COLOR);
    release_dc();
+   /* Unfortunately the background color produced by FillRect is slightly different
+    * from the desired background.  So patch it up with blank_rect.
+    */
+   blank_rect (0, 0, MSW_Screen_Cols, MSW_Screen_Rows);
    vterm_cls ();
 }
 
@@ -1132,16 +1215,41 @@ static void msw_get_terminfo (void)
    SLsmg_set_terminal_info (&tt);
 }
 
+static char *convert_color (char *c, char *buf)
+{
+   unsigned long ul;
+
+   if ((*c != '#') 
+       || (7 != strlen (c)))   /* e.g., #RRGGBB */
+     return c;
+   
+   if (1 != sscanf (c+1, "%lX", &ul))
+     return c;
+   
+   if (ul > 0xFFFFFFUL)
+     return c;
+
+   sprintf (buf, "%ld,%ld,%ld", (ul >> 16)&0xFF, (ul >> 8)&0xFF, ul & 0xFF);
+   return buf;
+}
+   
 
 static void msw_set_color (int i, char *what, char *fg, char *bg)
 {
    int r, g, b;
-   char buf[30];
+   char buf[64];
+   char fg_buf[64];
+   char bg_buf[64];
+   HBRUSH bgbr;
 
    if ((i < 0) || (i > JMAX_COLORS))
      return;
 
    (void) what;
+
+   fg = convert_color (fg, fg_buf);
+   bg = convert_color (bg, bg_buf);
+
    r = GetRValue(JColors[i].fg);
    g = GetGValue(JColors[i].fg);
    b = GetBValue(JColors[i].fg);
@@ -1172,8 +1280,12 @@ static void msw_set_color (int i, char *what, char *fg, char *bg)
 	release_dc();
      }
 #endif
-   DeleteObject(JColors[i].hbrBG);
-   JColors[i].hbrBG = CreateSolidBrush(JColors[i].bg);
+   bgbr = CreateSolidBrush(JColors[i].bg);
+   if (bgbr != 0)
+     {
+	DeleteObject(JColors[i].hbrBG);
+	JColors[i].hbrBG = bgbr;
+     }
 
 #if SLANG_VERSION >= 10306
    SLsmg_touch_screen ();
@@ -1271,30 +1383,8 @@ static void push_mouse_event(int button, int x, int y, int state, int type)
 	type = JMOUSE_IGNORE_EVENT;
      }
 #endif
-
-   switch (button)
-     {
-      case WM_LBUTTONUP:
-      case WM_LBUTTONDOWN:
-	if (!(GetKeyState(VK_MENU) & 0x8000))
-	  {
-	     jm.button = JMOUSE_BUTTON_1;
-	     break;
-	  }
-	/* drop */
-      case WM_MBUTTONUP:
-      case WM_MBUTTONDOWN:
-	jm.button = JMOUSE_BUTTON_2;
-	break;
-
-      case WM_RBUTTONUP:
-      case WM_RBUTTONDOWN:
-	jm.button = JMOUSE_BUTTON_3;
-	break;
-
-      default:
-	return;
-     }
+   
+   jm.button = button;
 
    if ((type == JMOUSE_UP)
        && (0 == (jm.state & jm.button)))
@@ -1324,6 +1414,35 @@ static void push_mouse_event(int button, int x, int y, int state, int type)
    _putkey(id);
 }
 
+static void push_wm_mouse_event (int button, int x, int y, int state, int type)
+{
+   switch (button)
+     {
+      case WM_LBUTTONUP:
+      case WM_LBUTTONDOWN:
+	if (!(GetKeyState(VK_MENU) & 0x8000))
+	  {
+	     button = JMOUSE_BUTTON_1;
+	     break;
+	  }
+	/* drop */
+      case WM_MBUTTONUP:
+      case WM_MBUTTONDOWN:
+	button = JMOUSE_BUTTON_2;
+	break;
+
+      case WM_RBUTTONUP:
+      case WM_RBUTTONDOWN:
+	button = JMOUSE_BUTTON_3;
+	break;
+
+      default:
+	return;
+     }
+   
+   push_mouse_event (button, x, y, state, type);
+}
+
 static void msw_define_color(char *color_name, int *pr, int *pg, int *pb)
 {
    char buf[30];
@@ -1387,7 +1506,7 @@ unsigned char sys_getkey(void)
      {
 	SLKeyBoard_Quit = 0;
 	flush_input ();
-	return Abort_Char;
+	return Jed_Abort_Char;
      }
 
    n = my_getkey();
@@ -1466,7 +1585,7 @@ unsigned char sys_getkey(void)
      {
 	SLKeyBoard_Quit = 0;
 	flush_input ();
-	return (unsigned char)Abort_Char;
+	return Jed_Abort_Char;
      }
 
    return my_getkey();
@@ -1552,6 +1671,19 @@ static LRESULT CALLBACK process_key_down (HWND hWnd, UINT msg, WPARAM wParam, LP
 	  }
 	return DefWindowProc(hWnd, msg, wParam, lParam);
 
+      /* PF1 and PF2 codes */
+      case 0x0145:
+	if (NumLock_Is_Gold)
+	  {
+	     buffer_keystring ("\000;", 2);
+	     Ignore_Wchar_Message = 1 ;
+	     return 0 ;
+	  }
+	return DefWindowProc(hWnd, msg, wParam, lParam);
+
+      case 0x0135:
+	c1 = 'Q'; break ;
+
       case 0xE02F:
 	c1 = 'Q'; break;	       /* KEYPAD SLASH */
 
@@ -1578,8 +1710,8 @@ static LRESULT CALLBACK process_key_down (HWND hWnd, UINT msg, WPARAM wParam, LP
       case 0x051:            /* KEYPAD PGDN */
       case 0x053:            /* KEYPAD DEL */
       case 0x052:            /* KEYPAD INSERT */
-	if (GetKeyState(VK_NUMLOCK) & 0x0001)
-	  return DefWindowProc(hWnd, msg, wParam, lParam);
+	if (!NumLock_Is_Gold && (GetKeyState(VK_NUMLOCK) & 0x0001))
+          return DefWindowProc(hWnd, msg, wParam, lParam);
 
  	state = 0;
  	if (key_state & KEY_SHIFT) state = 1;
@@ -1735,7 +1867,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 
       case WM_ERASEBKGND:
 	GetClientRect(This_Window.w, &rc);
-	FillRect((HDC)wParam, &rc, JColors[JNORMAL_COLOR].hbrBG);
+	FillRect((HDC)wParam, &rc, CURRENT_BG_COLOR);
 	break;
 
       case WM_SYSCHAR:
@@ -1746,11 +1878,22 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
       case WM_CHAR:
 	if (Ignore_Wchar_Message == 0)
 	  {
-	     if ((int) wParam == Abort_Char)
+	     if ((int) wParam == (int) Jed_Abort_Char)
 	       {
 		  if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 		  SLKeyBoard_Quit = 1;
 	       }
+	     /* PS 28Apr98: handle shiftTAB same as DOS */
+	     if (wParam == 0x009)
+	       {
+		  if (GetKeyState(VK_SHIFT)<0)
+		    {
+		       _putkey(0);
+		       _putkey('o' & 31);
+		       Ignore_Wchar_Message = 1;
+		       break;
+		    }
+	       }
 	     if (wParam == 0xE0) _putkey(wParam);
 	     _putkey(wParam);
 	     break;
@@ -1775,7 +1918,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 
 	if (!mouse_button_down) break;
 
-	push_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_UP);
+	push_wm_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_UP);
 	mouse_button_down--;
 	break;
 
@@ -1783,7 +1926,7 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
       case WM_MBUTTONDOWN:
       case WM_RBUTTONDOWN:
 	mouse_button_down++;
-	push_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
+	push_wm_mouse_event(msg, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
 	break;
 
       case WM_MOUSEMOVE:
@@ -1854,7 +1997,30 @@ LRESULT CALLBACK JEDWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
 	return 0;
 
       default:
-	return DefWindowProc(hWnd, msg, wParam, lParam);
+
+#if HAS_WHEEL_MOUSE_SUPPORT
+      /* Check for a mouse wheel movement message
+       * and translate it into a fake keystroke. This has now been
+       * tested on Win95,98,NT4,2000
+       */
+        if ((Wheel_Mouse.fActive 
+             && (msg == Wheel_Mouse.uiMshMsgMouseWheel))
+            || (msg == WM_MOUSEWHEEL))
+          {
+	     /* Under X, the wheel mouse produces JMOUSE_BUTTON_4 and
+ 	      * JMOUSE_BUTTON_5 events.  jed/lib/mouse.sl maps JMOUSE_BUTTON_4
+ 	      * events to upward movement and JMOUSE_BUTTON_5 to downward 
+ 	      * movement in the buffer.
+ 	      */
+	     if ((int) wParam < 0) 
+	       push_mouse_event (JMOUSE_BUTTON_5, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
+	     else 
+	       push_mouse_event (JMOUSE_BUTTON_4, LOWORD(lParam), HIWORD(lParam), wParam, JMOUSE_DOWN);
+
+ 	     return 0;
+          }
+#endif
+        return DefWindowProc(hWnd, msg, wParam, lParam);
      }
 
    return 0;
@@ -2381,7 +2547,42 @@ int jed_init_w32_support (void)
    return w32_init_subprocess_support (1);
 }
 
-#if defined(__BORLANDC__) || defined(__WIN32__)
+#if HAS_WHEEL_MOUSE_SUPPORT
+
+/* Initialise mouse wheel handling functionality */
+
+static int init_wheel_mouse (void)
+{
+   /* Find the mystery mouse window */
+   
+   Wheel_Mouse.hwnd = FindWindow(MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);
+
+   /* Register messages to determine mousey information */
+   
+   Wheel_Mouse.uiMshMsgMouseWheel  = RegisterWindowMessage(MSH_MOUSEWHEEL);
+   Wheel_Mouse.uiMshMsg3DSupport   = RegisterWindowMessage(MSH_WHEELSUPPORT);
+   Wheel_Mouse.uiMshMsgScrollLines = RegisterWindowMessage(MSH_SCROLL_LINES);
+
+   /* If we have a wheel enquiry message, ask about the presence of a wheel */
+   
+   if (Wheel_Mouse.uiMshMsg3DSupport)
+     Wheel_Mouse.fActive = (BOOL)SendMessage(Wheel_Mouse.hwnd, Wheel_Mouse.uiMshMsg3DSupport, 0, 0);
+   else
+     Wheel_Mouse.fActive = FALSE;  
+
+   /* If we have a scroll line enquiry message ask about that */
+   
+   if (Wheel_Mouse.uiMshMsgScrollLines)
+     Wheel_Mouse.iScrollLines = (int)SendMessage(Wheel_Mouse.hwnd, Wheel_Mouse.uiMshMsgScrollLines, 0, 0);
+   else
+     Wheel_Mouse.iScrollLines = 3;
+   
+   return 0;
+}
+#endif                                 /* HAS_WHEEL_MOUSE_SUPPORT */
+
+
+#if defined(__BORLANDC__) || defined(__WIN32__) || defined(__VC__)
 extern int main(int, char **);
 
 /* I do not think that this gets called.  Does it???? */
@@ -2405,6 +2606,10 @@ int PASCAL WinMain(HINSTANCE inst, HINSTANCE pinst, LPSTR lpszCmdLine, int nCmdS
 #endif
    (void) nCmdShow;
 
+#if HAS_WHEEL_MOUSE_SUPPORT
+   (void) init_wheel_mouse ();
+#endif
+
    strcpy(command_line, "wjed ");
    strcat(command_line, lpszCmdLine);
    _hPrev = pinst;
