@@ -65,6 +65,12 @@
 #include "file.h"
 #include "vterm.h"
 
+/* Fixes some X headers... */
+#ifndef Button6Mask
+# define Button6Mask (Button5Mask<<1)
+#endif
+
+
 /*}}}*/
 
 /*{{{ Static Global Variables */
@@ -1057,13 +1063,18 @@ static void fill_jmouse (JMouse_Type *jmouse, /*{{{*/
 
    if (button == Button1) jmouse->button = JMOUSE_BUTTON_1;
    else if (button == Button2) jmouse->button = JMOUSE_BUTTON_2;
-   else jmouse->button = JMOUSE_BUTTON_3;
+   else if (button == Button3) jmouse->button = JMOUSE_BUTTON_3;
+   else if (button == Button4) jmouse->button = JMOUSE_BUTTON_4;
+   else if (button == Button5) jmouse->button = JMOUSE_BUTTON_5;
+   else jmouse->button = JMOUSE_BUTTON_6;
 
    s = 0;
    if (state & Button1Mask) s |= JMOUSE_BUTTON_1;
    if (state & Button2Mask) s |= JMOUSE_BUTTON_2;
-   if (state & (Button3Mask|Button4Mask|Button5Mask))
-     s |= JMOUSE_BUTTON_3;
+   if (state & Button3Mask) s |= JMOUSE_BUTTON_3;
+   if (state & Button4Mask) s |= JMOUSE_BUTTON_4;
+   if (state & Button5Mask) s |= JMOUSE_BUTTON_5;
+   if (state & Button6Mask) s |= JMOUSE_BUTTON_5;
    if (state & ShiftMask) s |= JMOUSE_SHIFT;
    if (state & ControlMask) s |= JMOUSE_CTRL;
 
@@ -1227,7 +1238,7 @@ static int X_process_events (int force, char *buf, unsigned int buflen,
 	     if ((report.type == ButtonPress)
 		 && (0 == (report.xbutton.state
 			   & (Button1Mask|Button2Mask|Button3Mask
-			      |Button4Mask|Button5Mask))))
+			      |Button4Mask|Button5Mask|Button6Mask))))
 	       last_motion_down_button = report.xbutton.button;
 
 	     /* ESC ^@ is a  mouse prefix */
@@ -1463,7 +1474,7 @@ static void xjed_check_kbd(void) /*{{{*/
 	b = buf; bmax = b + n;
 	while (b < bmax)
 	  {
-	     if (*b == Abort_Char)
+	     if (*b == (char) Jed_Abort_Char)
 	       {
 		  if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 		  if (b != buf) buffer_keystring (buf, (int) (b - buf));
@@ -1938,16 +1949,16 @@ static int x_ioerr_handler (Display *d) /*{{{*/
 
 /*}}}*/
 
-/* returns socket descriptor */
-static int init_Xdisplay (void) /*{{{*/
+static int open_Xdisplay (void) /*{{{*/
 {
-   char dname[80], *n;
+   char dname [256];
+   char *n;
 
    n = X_Arg_List[XARG_DISPLAY].value;
    if (n != NULL)
      {
-	strncpy (dname, X_Arg_List[XARG_DISPLAY].value, 70);
-	dname[70] = 0;
+	strncpy (dname, X_Arg_List[XARG_DISPLAY].value, sizeof (dname)-10);
+	dname[sizeof(dname)-10] = 0;
 	n = dname;
 	while (*n && (*n != ':')) n++;
 	if (*n == 0) strcpy(n, ":0.0");
@@ -1955,16 +1966,18 @@ static int init_Xdisplay (void) /*{{{*/
      }
 
    XSetIOErrorHandler (x_ioerr_handler);
-
+   
    if ( (This_XDisplay = XOpenDisplay(n)) == NULL )
-     {
-  	(void) fprintf( stderr, "XJED: cannot connect to X server %s\n",
-		       XDisplayName(n));
-	exit( -1 );
-     }
+     return 0;
 
    XSetErrorHandler (x_err_handler);
+   return 1;
+}
 
+
+/* returns socket descriptor */
+static int init_Xdisplay (void) /*{{{*/
+{
 #ifdef XJED_USE_R6IM
    setlocale(LC_ALL, "");
 #endif
@@ -1983,11 +1996,11 @@ static int init_Xdisplay (void) /*{{{*/
 
    if (!load_font(XWin->font_name))
      {
-	(void) fprintf( stderr, "XJED: cannot load font %s, using fixed.\n", XWin->font_name);
+	(void) fprintf( stderr, "xjed: cannot load font %s, using fixed.\n", XWin->font_name);
 	if (!load_font("fixed"))
 	  {
-	     (void) fprintf( stderr, "XJED: cannot load fixed font.\n");
-	     exit( -1 );
+	     (void) fprintf( stderr, "xjed: cannot load fixed font.\n");
+	     exit (1);
 	  }
      }
 
@@ -2610,6 +2623,14 @@ static char *x_server_vendor (void)
    return The_Xserver_Vendor;
 }
 
+#if SLANG_VERSION < 10404
+static char *get_termcap_string (char *cap) /*{{{*/
+{
+   return "";
+}
+#endif
+
+
 static void x_set_meta_keys (int *maskp)
 {
    int mask = *maskp;
@@ -2668,6 +2689,9 @@ static SLang_Intrin_Fun_Type sl_x_table[] = /*{{{*/
     * This function returns the vendor name of the X server.
     */
    MAKE_INTRINSIC_I("x_set_meta_keys", x_set_meta_keys, SLANG_VOID_TYPE),
+#if SLANG_VERSION < 10404
+   MAKE_INTRINSIC_S("get_termcap_string", get_termcap_string, STRING_TYPE),
+#endif
    MAKE_INTRINSIC(NULL,NULL,0,0)
 };
 
@@ -2765,7 +2789,10 @@ static int JX_init_video (void) /*{{{*/
 
 void flush_output (void) /*{{{*/
 {
-   fflush (stdout);
+   if (This_XDisplay == NULL)
+     fflush (stdout);
+   else
+     SLtt_flush_output ();
 }
 
 /*}}}*/
@@ -2780,10 +2807,42 @@ static int JX_flush_output (void)
 
 static int JX_Zero = 0;
 
+static void get_screen_size (int *r, int *c)
+{
+   SLtt_get_screen_size ();
+   *r = SLtt_Screen_Rows;
+   *c = SLtt_Screen_Cols;
+}
+
+
 static void JX_get_terminfo (void) /*{{{*/
 {
    SLsmg_Term_Type tt;
 
+   if (!open_Xdisplay())
+    {
+       tt_beep                 = SLtt_beep;
+       tt_write_string         = SLtt_write_string;
+       tt_get_terminfo         = SLtt_get_terminfo;
+       tt_set_color            = SLtt_set_color;
+       tt_set_mono             = SLtt_set_mono;
+       tt_set_color_esc        = SLtt_set_color_esc;
+       tt_wide_width           = SLtt_wide_width;
+       tt_narrow_width         = SLtt_narrow_width;
+       tt_enable_cursor_keys   = SLtt_enable_cursor_keys;
+       tt_set_term_vtxxx       = SLtt_set_term_vtxxx;
+       tt_Ignore_Beep          = &SLtt_Ignore_Beep;
+       tt_Use_Ansi_Colors      = &SLtt_Use_Ansi_Colors;
+       tt_Term_Cannot_Scroll   = &SLtt_Term_Cannot_Scroll;
+       tt_Term_Cannot_Insert   = &SLtt_Term_Cannot_Insert;
+       tt_Blink_Mode           = &SLtt_Blink_Mode;
+       /*     tt_Baud_Rate            = &SLtt_Baud_Rate; */
+       tt_get_screen_size      = get_screen_size;
+
+       SLtt_get_terminfo ();
+       return;
+    }
+
    JX_Screen_Cols = 80;
    JX_Screen_Rows = 24;
    
